(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'core-decorators'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('core-decorators'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.coreDecorators);
    global.BetterStorage = mod.exports;
  }
})(this, function (exports, _coreDecorators) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.default = undefined;

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  var _class;

  var BetterStorage = (0, _coreDecorators.autobind)(_class = function () {
    function BetterStorage() {
      var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
      var kind = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'local';
      var overrides = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      _classCallCheck(this, BetterStorage);

      this.prefix = prefix;
      this.kind = kind;
      this.store = typeof kind === 'string' ? window[kind + 'Storage'] : kind;

      this.methods = Object.assign({
        doSet: this._doSet,
        doGet: this._doGet,
        doRemove: this._doRemove,
        doClear: this._doClear,
        doLength: this._doLength,
        doFullLength: this._doFullLength,
        doKey: this._doKey,
        doFullKey: this._doFullKey,
        doHas: this._doHas
      }, overrides);
    }

    _createClass(BetterStorage, [{
      key: '_doSet',
      value: function _doSet(key, data) {
        return this.store.setItem(key, JSON.stringify({ value: data }));
      }
    }, {
      key: '_doGet',
      value: function _doGet(key) {
        try {
          return JSON.parse().value;
        } catch (e) {
          return this.store.getItem(key);
        }
      }
    }, {
      key: '_doRemove',
      value: function _doRemove(key) {
        return this.store.removeItem(key);
      }
    }, {
      key: '_doClear',
      value: function _doClear() {
        return this.store.clear();
      }
    }, {
      key: '_doLength',
      value: function _doLength() {
        return this.keys.length;
      }
    }, {
      key: '_doFullLength',
      value: function _doFullLength() {
        return this.store.length;
      }
    }, {
      key: '_doKey',
      value: function _doKey(n) {
        return this.keys[n];
      }
    }, {
      key: '_doFullKey',
      value: function _doFullKey(n) {
        return this.store.key(n);
      }
    }, {
      key: '_doHas',
      value: function _doHas(key) {
        return this.fullKeys.indexOf(key) > -1;
      }
    }, {
      key: 'getStoreKey',
      value: function getStoreKey() {
        var key = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
        var prefix = arguments[1];

        prefix = prefix || this.prefix;
        if (prefix === '') {
          return key;
        }
        return prefix + '-' + key;
      }
    }, {
      key: 'setPrefix',
      value: function setPrefix() {
        var _this = this;

        var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
        var changeExisting = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        if (changeExisting && this.prefix !== '' && this.keys.length > 0) {
          this.keys.forEach(function (key) {
            var val = _this.store.getItem(_this.getStoreKey(key));
            _this.store.removeItem(_this.getStoreKey(key));
            _this.store.setItem(_this.getStoreKey(key, prefix), val);
          });
        }
        this.prefix = prefix;
      }
    }, {
      key: 'setKind',
      value: function setKind(kind) {
        var _this2 = this;

        var migrateData = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        var newStore = typeof kind === 'string' ? window[kind + 'Storage'] : kind;
        if (migrateData && this.prefix !== '' && this.keys.length > 0) {
          this.keys.forEach(function (key) {
            var val = _this2.store.getItem(_this2.getStoreKey(key));
            _this2.store.removeItem(_this2.getStoreKey(key));
            newStore.setItem(_this2.getStoreKey(key), val);
          });
        }
        this.kind = kind;
        this.store = newStore;
      }
    }, {
      key: 'set',
      value: function set(key, value) {
        return this.methods.doSet(this.getStoreKey(key), value);
      }
    }, {
      key: 'get',
      value: function get(key) {
        return this.methods.doGet(this.getStoreKey(key));
      }
    }, {
      key: 'remove',
      value: function remove(key) {
        return this.methods.doRemove(this.getStoreKey(key));
      }
    }, {
      key: 'has',
      value: function has(key) {
        return this.methods.doHas(this.getStoreKey(key));
      }
    }, {
      key: 'clear',
      value: function clear() {
        var _this3 = this;

        var full = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

        if (this.prefix === '' || full) {
          return this.methods.doClear();
        } else {
          return Promise.all(this.keys.map(function (key) {
            return _this3.methods.doRemove(_this3.getStoreKey(key));
          }));
        }
      }
    }, {
      key: 'getLength',
      value: function getLength() {
        return this.methods.doLength();
      }
    }, {
      key: 'getFullLength',
      value: function getFullLength() {
        return this.methods.doFullLength();
      }
    }, {
      key: 'key',
      value: function key(n) {
        return this.methods.doKey(n);
      }
    }, {
      key: 'fullKey',
      value: function fullKey(n) {
        return this.methods.doFullKey(n);
      }
    }, {
      key: 'keys',
      get: function get() {
        var _this4 = this;

        if (this.prefix === '') {
          return this.fullKeys;
        }
        return Object.keys(this.store).filter(function (item) {
          return item.indexOf(_this4.prefix + '-') === 0;
        }).map(function (item) {
          return item.replace(_this4.prefix + '-', '');
        });
      }
    }, {
      key: 'fullKeys',
      get: function get() {
        return Object.keys(this.store);
      }
    }]);

    return BetterStorage;
  }()) || _class;

  exports.default = BetterStorage;
});