(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'core-decorators'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('core-decorators'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.coreDecorators);
    global.BetterStorageAsync = mod.exports;
  }
})(this, function (exports, _coreDecorators) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.default = undefined;

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  var _class;

  function asPromsieResolve(val) {
    if (val instanceof Promise) {
      return val;
    }
    return new Promsie(function (accept) {
      return accept(val);
    });
  }

  var BetterStorageAsync = (0, _coreDecorators.autobind)(_class = function () {
    function BetterStorageAsync() {
      var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
      var kind = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'local';
      var overrides = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      _classCallCheck(this, BetterStorageAsync);

      this.prefix = prefix;
      this.kind = kind;
      this.store = typeof kind === 'string' ? window[kind + 'Storage'] : kind;

      this.methods = Object.assign({
        doSet: this._doSet,
        doGet: this._doGet,
        doRemove: this._doRemove,
        doClear: this._doClear,
        doLength: this._doLength,
        doFullLength: this._doFullLength,
        doKey: this._doKey,
        doFullKey: this._doFullKey,
        doHas: this._doHas
      }, overrides);
    }

    _createClass(BetterStorageAsync, [{
      key: '_doSet',
      value: function _doSet(key, data) {
        return asPromsieResolve(this.store.setItem(key, JSON.stringify({ value: data })));
      }
    }, {
      key: '_doGet',
      value: function _doGet(key) {
        var _this = this;

        return new Promise(function (accept, reject) {
          asPromsieResolve(_this.store.getItem(key)).then(function (data) {
            try {
              accept(JSON.parse(data).value);
            } catch (e) {
              accept(data);
            }
          }).catch(reject);
        });
      }
    }, {
      key: '_doRemove',
      value: function _doRemove(key) {
        return asPromsieResolve(this.store.removeItem(key));
      }
    }, {
      key: '_doClear',
      value: function _doClear() {
        return asPromsieResolve(this.store.clear());
      }
    }, {
      key: '_doLength',
      value: function _doLength() {
        var _this2 = this;

        return new Promise(function (accept, reject) {
          _this2.keys.then(function (keys) {
            return accept(keys.length);
          }).catch(reject);
        });
      }
    }, {
      key: '_doFullLength',
      value: function _doFullLength() {
        var _this3 = this;

        return new Promise(function (accept, reject) {
          _this3.fullKeys.then(function (keys) {
            return accept(keys.length);
          }).catch(reject);
        });
      }
    }, {
      key: '_doKey',
      value: function _doKey(n) {
        var _this4 = this;

        return new Promise(function (accept, reject) {
          _this4.keys.then(function (keys) {
            return accept(keys[n]);
          }).catch(reject);
        });
      }
    }, {
      key: '_doFullKey',
      value: function _doFullKey(n) {
        var _this5 = this;

        return new Promise(function (accept, reject) {
          _this5.fullKeys.then(function (keys) {
            return accept(keys[n]);
          }).catch(reject);
        });
      }
    }, {
      key: '_doHas',
      value: function _doHas(key) {
        var _this6 = this;

        return new Promise(function (accept, reject) {
          _this6.fullKeys.then(function (keys) {
            return accept(keys.indexOf(key) > -1);
          }).catch(reject);
        });
      }
    }, {
      key: 'getStoreKey',
      value: function getStoreKey() {
        var key = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
        var prefix = arguments[1];

        prefix = prefix || this.prefix;
        if (prefix === '') {
          return key;
        }
        return prefix + '-' + key;
      }
    }, {
      key: 'setPrefix',
      value: function setPrefix() {
        var _this7 = this;

        var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
        var changeExisting = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        return new Promise(function (accept, reject) {
          if (changeExisting && _this7.prefix !== '' && _this7.keys.length > 0) {
            _this7.keys.then(function (keys) {
              Promise.all(keys.map(function (key) {
                return new Promise(function (acceptInner, rejectInner) {
                  _this7.store.getItem(_this7.getStoreKey(key)).then(function (value) {
                    return acceptInner({ key: key, value: value });
                  }).catch(rejectInner);
                });
              })).then(function (pairs) {
                Promise.all(pairs.reduce(function (promises, data) {
                  promises.push(_this7.store.removeItem(_this7.getStoreKey(data.key)));
                  promises.push(_this7.store.setItem(_this7.getStoreKey(data.key, prefix), data.value));
                  return promises;
                }, [])).then(function () {
                  _this7.prefix = prefix;
                  accept();
                }).catch(reject);
              }).catch(reject);
            }).catch(reject);
          } else {
            _this7.prefix = prefix;
            accept();
          }
        });
      }
    }, {
      key: 'setKind',
      value: function setKind(kind) {
        var _this8 = this;

        var migrateData = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        return new Promise(function (accept, reject) {
          var newStore = typeof kind === 'string' ? window[kind + 'Storage'] : kind;
          if (migrateData && _this8.prefix !== '' && _this8.keys.length > 0) {
            _this8.keys.then(function (keys) {
              Promise.all(keys.map(function (key) {
                return new Promise(function (acceptInner, rejectInner) {
                  _this8.store.getItem(_this8.getStoreKey(key)).then(function (value) {
                    return acceptInner({ key: key, value: value });
                  }).catch(rejectInner);
                });
              })).then(function (pairs) {
                Promise.all(pairs.reduce(function (promises, data) {
                  promises.push(_this8.store.removeItem(_this8.getStoreKey(data.key)));
                  promises.push(newStore.setItem(_this8.getStoreKey(data.key), data.value));
                  return promises;
                }, [])).then(function () {
                  _this8.kind = kind;
                  _this8.store = newStore;
                  accept();
                }).catch(reject);
              }).catch(reject);
            }).catch(reject);
          } else {
            _this8.kind = kind;
            _this8.store = newStore;
            accept();
          }
        });
      }
    }, {
      key: 'set',
      value: function set(key, value) {
        return this.methods.doSet(this.getStoreKey(key), value);
      }
    }, {
      key: 'get',
      value: function get(key) {
        return this.methods.doGet(this.getStoreKey(key));
      }
    }, {
      key: 'remove',
      value: function remove(key) {
        return this.methods.doRemove(this.getStoreKey(key));
      }
    }, {
      key: 'has',
      value: function has(key) {
        return this.methods.doHas(this.getStoreKey(key));
      }
    }, {
      key: 'clear',
      value: function clear() {
        var _this9 = this;

        var full = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

        if (this.prefix === '' || full) {
          return this.methods.doClear();
        }
        return new Promise(function (accept, reject) {
          _this9.keys.then(function (keys) {
            Promise.all(keys.map(function (key) {
              return _this9.methods.doRemove(_this9.getStoreKey(key));
            })).then(accept).catch(reject);
          }).catch(reject);
        });
      }
    }, {
      key: 'getLength',
      value: function getLength() {
        return this.methods.doLength();
      }
    }, {
      key: 'getFullLength',
      value: function getFullLength() {
        return this.methods.doFullLength();
      }
    }, {
      key: 'key',
      value: function key(n) {
        return this.methods.doKey(n);
      }
    }, {
      key: 'fullKey',
      value: function fullKey(n) {
        return this.methods.doFullKey(n);
      }
    }, {
      key: 'keys',
      get: function get() {
        var _this10 = this;

        if (this.prefix === '') {
          return this.fullKeys;
        }
        return new Promise(function (resolve, reject) {
          _this10.fullKeys.then(function (keys) {
            resolve(keys.filter(function (item) {
              return item.indexOf(_this10.prefix + '-') === 0;
            }).map(function (item) {
              return item.replace(_this10.prefix + '-', '');
            }));
          }).catch(reject);
        });
      }
    }, {
      key: 'fullKeys',
      get: function get() {
        var _this11 = this;

        if (this.store.hasOwnProperty('getAllKeys')) {
          return this.store.getAllKeys();
        }
        return new Promise(function (accept) {
          return accept(Object.keys(_this11.store));
        });
      }
    }]);

    return BetterStorageAsync;
  }()) || _class;

  exports.default = BetterStorageAsync;
});